#!/usr/bin/env python

import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
import matplotlib.path as mpath
import numpy as np
import numpy.ma as ma
import netCDF4
import cartopy
import cartopy.crs as ccrs
import string
import matplotlib as mpl
from scipy.stats import norm
from matplotlib import cm

# Figure 13: ENSO
#==================
# Load datasets (Monthly intantaneous snapshots)
f1 = netCDF4.Dataset('L35_Ts_ENSO_yrs1600-1950_GMDplot.nc')
# Grid info
lons = f1.variables['longitude'][:]
lats = f1.variables['latitude'][:]
# De-seasoned SST over tropical Pacific
tsp  = f1.variables['Ts'][:,:,:]
f1.close()

f1 = netCDF4.Dataset('L20_Ts_ENSO_yrs1600-1950_GMDplot.nc')
# De-seasoned SST over tropical Pacific
tsp_20  = f1.variables['Ts'][:,:,:]
f1.close()

n3p4 = np.mean(np.mean(tsp[:,9:13,28:46],axis=1),axis=1)
n3p4_20 = np.mean(np.mean(tsp_20[:,9:13,28:46],axis=1),axis=1)

# N3.4 obs from HadISST
#==================
lines = open('nino34.long.data').readlines()
X     = []
for line in lines:
    s = str.split(line)
    X.append([float(s[i]) for i in range(1,len(s))])
Y = np.asarray(X)
Ya = Y-np.mean(Y,axis=0) # Deseason
N34=np.reshape(Ya,(Ya.shape[0]*Y.shape[1],1))
N34a = N34-np.mean(N34)

# Compute PDF
#==================
xmin, xmax = -3,3#plt.xlim()
#print(xmin,xmax)
x = np.linspace(xmin, xmax, 100)
muH, stdH = norm.fit(N34a)
p = norm.pdf(x, muH, stdH)
mu35, std35 = norm.fit(n3p4)
p35 = norm.pdf(x, mu35, std35)
mu20, std20 = norm.fit(n3p4_20)
p20 = norm.pdf(x, mu20, std20)

#==================
def ENSOFglobarr( ax, inarray, cmin=None, cmax=None, pal='seismic', clev=6 , clab=6):

    ax.set_extent([111, 288, -30, 30], crs=ccrs.PlateCarree())
    ax.coastlines()
    ax.gridlines()
    ax.add_feature(cartopy.feature.LAND, zorder=0)

    cs = ax.pcolormesh(lons ,lats, inarray, cmap=pal, vmin=cmin, vmax=cmax, transform=ccrs.PlateCarree())
#==================

# Define composites for El Nino and La Nina states, where nino3.4 anomalies exceed +/- 1 std
idxp = np.where(n3p4 > n3p4*0+std35)
idxn = np.where(n3p4 < n3p4*0-std35)
npm = np.mean(tsp[idxp[0],:,:],axis=0)
nnm = np.mean(tsp[idxn[0],:,:],axis=0)

clev=6
clab=6
cy = np.linspace(0,4200,4200)/12

fig = plt.figure(figsize=(14,14))

#                    X,    Y,    dx,   dy 
ax0 = fig.add_axes([0.00, 0.70, 0.80, 0.30],projection=ccrs.Mercator(central_longitude=-180.0, min_latitude=-30.0, max_latitude=30.0, 
                                                globe=None, latitude_true_scale=0.0))
ax1 = fig.add_axes([0.00, 0.40, 0.80, 0.30],projection=ccrs.Mercator(central_longitude=-180.0, min_latitude=-30.0, max_latitude=30.0, 
                                                globe=None, latitude_true_scale=0.0))
axC = fig.add_axes([0.69, 0.25, 0.17, 0.90])

ax3 = fig.add_axes([0.00, 0.21, 0.90, 0.16])
ax4 = fig.add_axes([0.00, 0.00, 0.90, 0.16])

ENSOFglobarr( ax0, npm, cmin=-1.5, cmax=1.5, pal="RdBu_r", clev=clev, clab=clab)
ax0.text(-0.05, 1.05, '(a)', transform=ax0.transAxes, size=20)

ENSOFglobarr( ax1, nnm, cmin=-1.5, cmax=1.5, pal="RdBu_r", clev=clev, clab=clab)
ax1.text(-0.05, 1.05, '(b)', transform=ax1.transAxes, size=20)

norm = mpl.colors.Normalize(vmin=-1.5, vmax=1.5)
cbar = axC.figure.colorbar(cm.ScalarMappable(norm=norm, cmap="RdBu_r"), ax=axC,shrink=.98,pad=0.02, extend='both')
cbar.ax.tick_params(labelsize=14) 
axC.axis('off')

npstd = np.std(n3p4)
ax3.plot(cy,n3p4)
ax3.fill_between(cy,n3p4*0-npstd,n3p4*0+npstd,alpha=0.5,color="grey")
ax3.set_xlabel("Time (years)", fontsize=18)
ax3.set_ylabel("$\Delta$T ($^o$C)", fontsize=18)
ax3.set_ylim(-3,3)
ax3.tick_params(labelsize=16)
ax3.text(-0.05, 1.05, '(c)', transform=ax3.transAxes, size=20)

#ax4
ax4.hist(n3p4, bins=39, range=[-3,3],density=True, alpha=0.6, color='royalblue')
ax4.plot(x, p, 'k', linewidth=2,label="HadISST (%.2f)" % (stdH))
ax4.plot(x, p35, 'b', linewidth=2,label=" L35 (%.2f)" % (std35))
ax4.plot(x, p20, 'r', linewidth=2,label=" L20 (%.2f)" % (std20))
ax4.set_xlabel("$\Delta$T ($^o$C)", fontsize=18)
ax4.legend(fontsize=18)
ax4.text(-0.05, 1.05, '(d)', transform=ax4.transAxes, size=20);
ax4.tick_params(labelsize=16)

plt.savefig("Figures/FORTE2_Figure13_R1.pdf", dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)


#Figure 15: NAO
#==================
# Load datasets (Daily data)
f1 = netCDF4.Dataset('FORTE_NAO_GMDplot.nc')
f2 = netCDF4.Dataset('C20th_NAO_GMDplot.nc')

# Grid info
lon  = f1.variables['longitude'][:]
lat  = f1.variables['latitude'][:]
pcs  = f1.variables['pcs'][:]
eofs = f1.variables['eofs'][:]
evs  = f1.variables['evs'][:]


olon  = f2.variables['longitude'][:]
olat  = f2.variables['latitude'][:]
opcs  = f2.variables['pcs'][:]
oeofs = f2.variables['eofs'][:]
oevs  = f2.variables['evs'][:]

print("FORTE EOF variances",evs[:5]*100)
print("Obs   EOF variances",oevs[:5]*100)

# Create figure
#==================
def Fglobarr( ax, ilon, ilat, inarray, cmin=None, cmax=None, pal='seismic', clev=6 , clab=6):

    ax.set_global()
    ax.set_extent([-90, 40, 20, 80], crs=ccrs.PlateCarree())
    ax.coastlines()
    ax.gridlines()
    ax.add_feature(cartopy.feature.LAND, zorder=0)
    # Make a boundary path in PlateCarree projection, I choose to start in
    # the bottom left and go round anticlockwise, creating a boundary point
    # every 1 degree so that the result is smooth:
    vertices = [(lon, 20) for lon in range(-90, 41, 1)] +                [(lon, 80) for lon in range(40, -81, -1)]
    boundary = mpath.Path(vertices)
    ax.set_boundary(boundary, transform=ccrs.PlateCarree())

    cs = ax.pcolormesh(ilon ,ilat, inarray, cmap=pal, vmin=cmin, vmax=cmax, transform=ccrs.PlateCarree())
    cbar = plt.colorbar(mappable=cs,orientation='horizontal',ax=ax,shrink=.6,pad=0.05, extend='both')
    cbar.ax.tick_params(labelsize=20) 
#==================

fig = plt.figure(figsize=(21,10))

comp=0
# Figure 1a
clev = [-2,2,6,10,14,18,22,26]
clab = [2,10,18,26]
ax1 = plt.subplot2grid((3, 4), (0, 0),colspan=2,rowspan=2, projection=ccrs.AlbersEqualArea(central_longitude=-20.0, central_latitude=50.0, 
                                                                false_easting=0.0, false_northing=0.0, 
                                                                standard_parallels=(20.0, 50.0), globe=None))
Fglobarr( ax1, lon, lat, eofs[comp,:,:], cmin=-4, cmax=4, pal="RdBu_r", clev=clev, clab=clab)
ax1.text(-0.05, 1.01, '(a)', transform=ax1.transAxes, size=20)#, weight='bold')
ax1.text(0.5, 1.0, "    {:.2f}%".format(evs[comp]*100), transform=ax1.transAxes, size=20, ha='center')

# Figure 1b
ax2 = plt.subplot2grid((3, 4), (2, 0),colspan=2)
plt.plot(pcs[:,comp]*0,color='k')
plt.plot(pcs[:,comp],color='b')
ax2.tick_params(labelsize=18)
ax2.set_xlabel("Time (years)", fontsize=20)
ax2.set_ylabel("NAO Index", fontsize=20)
ax2.set_ylim(-3,3)
ax2.text(-0.05, 1.08, '(b)', transform=ax2.transAxes, size=20)#, weight='bold')

# Figure 1c
clev = [-2,2,6,10,14,18,22,26]
clab = [2,10,18,26]
ax3 = plt.subplot2grid((3, 4), (0, 2),colspan=2,rowspan=2, projection=ccrs.AlbersEqualArea(central_longitude=-20.0, central_latitude=50.0, 
                                                                false_easting=0.0, false_northing=0.0, 
                                                                standard_parallels=(20.0, 50.0), globe=None))
Fglobarr( ax3, olon, olat, oeofs[comp,:,:], cmin=-4, cmax=4, pal="RdBu_r", clev=clev, clab=clab)
ax3.text(-0.05, 1.01, '(c)', transform=ax3.transAxes, size=20)#, weight='bold')
ax3.text(0.5, 1.0, "    {:.2f}%".format(oevs[comp]*100), transform=ax3.transAxes, size=20, ha='center')

# Figure 1d
ax4 = plt.subplot2grid((3, 4), (2, 2),colspan=2)
plt.plot(opcs[:,comp]*0,color='k')
plt.plot(opcs[:,comp],color='b')
ax4.tick_params(labelsize=18)
ax4.set_xlabel("Time (years)", fontsize=20)
ax4.set_ylim(-3,3)
ax4.text(-0.05, 1.08, '(d)', transform=ax4.transAxes, size=20)#, weight='bold')

plt.savefig("Figures/FORTE2_Figure15_R1.pdf", dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)


# Figure 14: AO (Daily data)
#==================
# Load datasets (Daily data)
f1 = netCDF4.Dataset('FORTE_AO_GMDplot.nc')
f2 = netCDF4.Dataset('C20th_AO_GMDplot.nc')

# Grid info
lon  = f1.variables['longitude'][:]
lat  = f1.variables['latitude'][:]
pcs  = f1.variables['pcs'][:]
eofs = f1.variables['eofs'][:]
evs  = f1.variables['evs'][:]

olon  = f2.variables['longitude'][:]
olat  = f2.variables['latitude'][:]
opcs  = f2.variables['pcs'][:]
oeofs = f2.variables['eofs'][:]
oevs  = f2.variables['evs'][:]

print("FORTE EOF variances",evs[:5]*100)
print("Obs   EOF variances",oevs[:5]*100)

# Create figure
#==================
def Fglobarr( ax, ilon, ilat, inarray, cmin=None, cmax=None, pal='seismic', clev=6 , clab=6):

    from cartopy.util import add_cyclic_point

    ax.set_global()
    ax.coastlines()
    ax.gridlines()
    ax.add_feature(cartopy.feature.LAND, zorder=0)
    ilonsc = add_cyclic_point(ilon)
    inarrayc = add_cyclic_point(inarray)

    cs = ax.pcolormesh(ilonsc ,ilat, inarrayc, cmap=pal, vmin=cmin, vmax=cmax, transform=ccrs.PlateCarree())
    cbar = plt.colorbar(mappable=cs,orientation='horizontal',ax=ax,shrink=.6,pad=0.02, extend='both')
    cbar.ax.tick_params(labelsize=20) 
#==================

fig = plt.figure(figsize=(21,14))

comp=0
# Figure 1a
clev = [-2,2,6,10,14,18,22,26]
clab = [2,10,18,26]
ax1 = plt.subplot2grid((5, 4), (0, 0),colspan=2,rowspan=4, projection=ccrs.Orthographic(central_latitude=90.0, central_longitude=-20.0))
Fglobarr( ax1, lon, lat, eofs[comp,:,:], cmin=-4, cmax=4, pal="RdBu_r", clev=clev, clab=clab)
ax1.text(-0.05, 1.01, '(a)', transform=ax1.transAxes, size=20)#, weight='bold')
ax1.text(0.5, 1.02, " {:.2f}%".format(evs[comp]*100), transform=ax1.transAxes, size=20, ha='center')

# Figure 1b
ax2 = plt.subplot2grid((5, 4), (4, 0),colspan=2)
plt.plot(pcs[:,comp]*0,color='k')
plt.plot(pcs[:,comp],color='b')
ax2.tick_params(labelsize=18)
ax2.set_xlabel("Time (years)", fontsize=20)
ax2.set_ylabel("AO Index", fontsize=20)
ax2.set_ylim(-3,3)
ax2.text(-0.05, 1.08, '(b)', transform=ax2.transAxes, size=20)#, weight='bold')

# Figure 1c
clev = [-2,2,6,10,14,18,22,26]
clab = [2,10,18,26]
ax3 = plt.subplot2grid((5, 4), (0, 2),colspan=2,rowspan=4, projection=ccrs.Orthographic(central_latitude=90.0, central_longitude=-20.0))
Fglobarr( ax3, olon, olat, oeofs[comp,:,:], cmin=-4, cmax=4, pal="RdBu_r", clev=clev, clab=clab)
ax3.text(-0.05, 1.01, '(c)', transform=ax3.transAxes, size=20)#, weight='bold')
ax3.text(0.5, 1.02, " {:.2f}%".format(oevs[comp]*100), transform=ax3.transAxes, size=20, ha='center')

# Figure 1d
ax4 = plt.subplot2grid((5, 4), (4, 2),colspan=2)
plt.plot(opcs[:,comp]*0,color='k')
plt.plot(opcs[:,comp],color='b')
ax4.tick_params(labelsize=18)
ax4.set_xlabel("Time (years)", fontsize=20)
ax4.set_ylim(-3,3)
ax4.text(-0.05, 1.08, '(d)', transform=ax4.transAxes, size=20)#, weight='bold')

plt.savefig("Figures/FORTE2_Figure14_R1.pdf", dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)


# Figure 16: SAM (Daily data)
#==================
# Load datasets (Daily data)
f1 = netCDF4.Dataset('FORTE_SAM_GMDplot.nc')
f2 = netCDF4.Dataset('C20th_SAM_GMDplot.nc')

# Grid info
lon  = f1.variables['longitude'][:]
lat  = f1.variables['latitude'][:]
pcs  = f1.variables['pcs'][:]
eofs = f1.variables['eofs'][:]
evs  = f1.variables['evs'][:]

olon  = f2.variables['longitude'][:]
olat  = f2.variables['latitude'][:]
opcs  = f2.variables['pcs'][:]
oeofs = f2.variables['eofs'][:]
oevs  = f2.variables['evs'][:]

print("FORTE EOF variances",evs[:5]*100)
print("Obs   EOF variances",oevs[:5]*100)

# Create figure: FORTE SAM
#==================
def Fglobarr( ax, ilon, ilat, inarray, cmin=None, cmax=None, pal='seismic', clev=6 , clab=6):

    from cartopy.util import add_cyclic_point

    ax.set_global()
    ax.coastlines()
    ax.gridlines()
    ax.add_feature(cartopy.feature.LAND, zorder=0)
    ilonsc = add_cyclic_point(ilon)
    inarrayc = add_cyclic_point(inarray)

    cs = ax.pcolormesh(ilonsc ,ilat, inarrayc, cmap=pal, vmin=cmin, vmax=cmax, transform=ccrs.PlateCarree())
    cbar = plt.colorbar(mappable=cs,orientation='horizontal',ax=ax,shrink=.6,pad=0.02, extend='both')
    cbar.ax.tick_params(labelsize=20) 
#==================

fig = plt.figure(figsize=(21,14))

comp=0
# Figure 1a
clev = [-2,2,6,10,14,18,22,26]
clab = [2,10,18,26]
ax1 = plt.subplot2grid((5, 4), (0, 0),colspan=2,rowspan=4, projection=ccrs.Orthographic(central_latitude=-90.0, central_longitude=-20.0))
Fglobarr( ax1, lon, lat, eofs[comp,:,:]*-1, cmin=-5, cmax=5, pal="RdBu_r", clev=clev, clab=clab)
ax1.text(-0.05, 1.01, '(a)', transform=ax1.transAxes, size=20)#, weight='bold')
ax1.text(0.5, 1.02, " {:.2f}%".format(evs[comp]*100), transform=ax1.transAxes, size=20, ha='center')

# Figure 1b
ax2 = plt.subplot2grid((5, 4), (4, 0),colspan=2)
plt.plot(pcs[:,comp]*0,color='k')
plt.plot(pcs[:,comp]*-1,color='b')
ax2.tick_params(labelsize=18)
ax2.set_xlabel("Time (months)", fontsize=20)
ax2.set_ylabel("SAM Index", fontsize=20)
ax2.text(-0.05, 1.08, '(b)', transform=ax2.transAxes, size=20)#, weight='bold')

# Figure 1c
clev = [-2,2,6,10,14,18,22,26]
clab = [2,10,18,26]
ax3 = plt.subplot2grid((5, 4), (0, 2),colspan=2,rowspan=4, projection=ccrs.Orthographic(central_latitude=-90.0, central_longitude=-20.0))
Fglobarr( ax3, olon, olat, oeofs[comp,:,:]*-1, cmin=-5, cmax=5, pal="RdBu_r", clev=clev, clab=clab)
ax3.text(-0.05, 1.01, '(c)', transform=ax3.transAxes, size=20)#, weight='bold')
ax3.text(0.5, 1.02, " {:.2f}%".format(oevs[comp]*100), transform=ax3.transAxes, size=20, ha='center')

# Figure 1d
ax4 = plt.subplot2grid((5, 4), (4, 2),colspan=2)
plt.plot(opcs[:,comp]*0,color='k')
plt.plot(opcs[:,comp]*-1,color='b')
ax4.tick_params(labelsize=18)
ax4.set_xlabel("Time (months)", fontsize=20)
ax4.text(-0.05, 1.08, '(d)', transform=ax4.transAxes, size=20)#, weight='bold')

plt.savefig("Figures/FORTE2_Figure16_R1.pdf", dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)
