#!/usr/bin/env python

import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
import numpy as np
import numpy.ma as ma
import netCDF4
import cartopy
import cartopy.crs as ccrs
import string

def make_colormap(seq):
    """Return a LinearSegmentedColormap
    seq: a sequence of floats and RGB-tuples. The floats should be increasing
    and in the interval (0,1).
    """
    seq = [(None,) * 3, 0.0] + list(seq) + [1.0, (None,) * 3]
    cdict = {'red': [], 'green': [], 'blue': []}
    for i, item in enumerate(seq):
        if isinstance(item, float):
            r1, g1, b1 = seq[i - 1]
            r2, g2, b2 = seq[i + 1]
            cdict['red'].append([item, r1, r2])
            cdict['green'].append([item, g1, g2])
            cdict['blue'].append([item, b1, b2])
    return mcolors.LinearSegmentedColormap('CustomMap', cdict)


#== L35 ==================
# Load datasets
f1 = netCDF4.Dataset('L35_ocn_diags.mpicontrol_d30_GMDplot.oc.nc')

# Grid info
dp      = f1.variables['DEPTH'][:]/100
yr      = f1.variables['DAY'][:]/360

# Figure 1
hflux   = f1.variables['HFLUX'][:]
wflux   = f1.variables['WFLUX'][:]*1000
sst     = f1.variables['SST'][:]-273.15
sss     = f1.variables['SSS'][:]
tav     = f1.variables['TAV'][:]-273.15
sav     = f1.variables['SAV'][:]

# Figure 2
ttdsg   = f1.variables['TTDSG'][:,:]-273.15
stdsg   = f1.variables['STDSG'][:,:]
zsst    = f1.variables['ZSST'][:,0,:]-273.15
zsss    = f1.variables['ZSSS'][:,0,:]

# Figure 3
omax30  = f1.variables['OMAX30'][:]
dptrans = f1.variables['DRAKE_PASSAGE'][:]

# Figure 9
amoc    = f1.variables['AMOC'][:,:,:]

f1.close()
#=========================
#== L20 ==================
# Load datasets
f1 = netCDF4.Dataset('L20_ocn_diags.mpicontrol_d30_GMDplot.oc.nc')

# Grid info
dp20      = f1.variables['DEPTH'][:]/100
yr20      = f1.variables['DAY'][:]/360

# Figure 1
hflux20   = f1.variables['HFLUX'][:]
wflux20   = f1.variables['WFLUX'][:]*1000
sst20     = f1.variables['SST'][:]-273.15
sss20     = f1.variables['SSS'][:]
tav20     = f1.variables['TAV'][:]-273.15
sav20     = f1.variables['SAV'][:]

# Figure 2
ttdsg20   = f1.variables['TTDSG'][:,:]-273.15
stdsg20   = f1.variables['STDSG'][:,:]
zsst20    = f1.variables['ZSST'][:,0,:]-273.15
zsss20    = f1.variables['ZSSS'][:,0,:]

# Figure 3
omax3020  = f1.variables['OMAX30'][:]
dptrans20 = f1.variables['DRAKE_PASSAGE'][:]

# Figure 9
amoc20    = f1.variables['AMOC'][:,:,:]

f1.close()
#=========================

#== L35 and L20 ==================
f1 = netCDF4.Dataset('FORTE_SST_SSS_ZOS_GMDplot.nc')

lon     = f1.variables['longitude'][:]
lat     = f1.variables['latitude'][:]
lonu    = f1.variables['longitude_u'][:]
latu    = f1.variables['latitude_u'][:]

# Figure 7
sst2d   = f1.variables['SST35'][:]
sss2d   = f1.variables['SSS35'][:]
sst2d20 = f1.variables['SST20'][:]
sss2d20 = f1.variables['SSS20'][:]

# Figure 8
ssh     = f1.variables['ZOS35'][:]
ssh20   = f1.variables['ZOS20'][:]

f1.close()

# Figure 7 - Obs
f1      = netCDF4.Dataset('EN3_v2a_ObjectiveAnalysis_1960_1989_mean_on_FORTE_grid_GMDplot.nc')

elon    = f1.variables['lon'][:]
elat    = f1.variables['lat'][:]
esst2d  = np.mean(f1.variables['temperature'][:,0,:,:],axis=0)-273.15
esss2d  = np.mean(f1.variables['salinity'][:,0,:,:],axis=0)
f1.close()

# Figure 8 - Obs
f1      = netCDF4.Dataset('OCCAv2_SSH_on_FORTE_grid_GMDplot.nc')

lonssh  = f1.variables['lon'][:]
latssh  = f1.variables['lat'][:]
occassh = f1.variables['etan'][0,:,:]
f1.close()
#=========================


#=========================
# Figures 4 and 5
#=========================
#== L35 ==================
f1      = netCDF4.Dataset('L35_Ts_Precip_MSLP_720001_729000_GMDplot.nc')
# Grid info
lona    = np.append(f1.variables['longitude'][:],359.99)
lata   = np.insert(f1.variables['latitude'][:],0,90)

STanmn  = f1.variables['STanmn'][:,:]
STmonclim  = f1.variables['STmonclim'][:,:,:]
PTanmn  = f1.variables['PTanmn'][:,:]
PTmonclim  = f1.variables['PTmonclim'][:,:,:]
PSanmn  = f1.variables['PSanmn'][:,:]
PSmonclim  = f1.variables['PSmonclim'][:,:,:]
C20_SAT_monclim = f1.variables['C20_SAT_monclim'][:,:,:]
C20_Precip_monclim = f1.variables['C20_Precip_monclim'][:,:,:]

C20_SAT_clim = np.mean(C20_SAT_monclim,axis=0)
dSAT35 = STanmn - C20_SAT_clim

PTjjaclim  = np.mean(PTmonclim[6:8,:,:],axis=0)
PTdjfclim  = np.mean(np.roll(PTmonclim,1)[:3,:,:],axis=0)
PTdjfclim2 = np.roll(PTmonclim,1,axis=0)[:3,:,:]
PTjjaclim2 = PTmonclim[5:8,:,:]
C20_Precip_jjaclim = np.mean(C20_Precip_monclim[5:8,:,:],axis=0)
C20_Precip_djfclim = np.mean(np.roll(C20_Precip_monclim,1)[:3,:,:],axis=0)

f1.close()
#=========================
#== L20 ==================
f1      = netCDF4.Dataset('L20_Ts_Precip_MSLP_720001_729000_GMDplot.nc')
# Grid info
lona20    = np.append(f1.variables['longitude'][:],359.99)
lata20   = np.insert(f1.variables['latitude'][:],0,90)
press20  = f1.variables['pressure'][:]

STanmn20  = f1.variables['STanmn'][:,:]
STmonclim20  = f1.variables['STmonclim'][:,:,:]
PTanmn20  = f1.variables['PTanmn'][:,:]
PTmonclim20  = f1.variables['PTmonclim'][:,:,:]
PSanmn20  = f1.variables['PSanmn'][:,:]
PSmonclim20  = f1.variables['PSmonclim'][:,:,:]

PTjjaclim20  = np.mean(PTmonclim20[6:8,:,:],axis=0)
PTdjfclim20  = np.mean(np.roll(PTmonclim20,1)[:3,:,:],axis=0)
PTdjfclim220 = np.roll(PTmonclim20,1,axis=0)[:3,:,:]
PTjjaclim220 = PTmonclim20[5:8,:,:]

f1.close()
#=========================
# Figure 6
#=========================
#== L35 ==================
f1      = netCDF4.Dataset('u_zm.nc')
pres    = f1.variables['Pressure'][:]
lata2   = f1.variables['Latitude'][:]
UJJA    = f1.variables['UJJA'][:,:]
UDJF    = f1.variables['UDJF'][:,:]
f1.close()
#=========================
#== L20 ==================
d1      = '/noc/msm/working/valor/New_FORTE/L20/u_djf_jja_64x20_lines.dat'
x = np.loadtxt(d1, delimiter=',', dtype=float)
UDJF20    = x[:20,:]
UJJA20    = x[20:,:]
#=========================


#=========================
# Figure 10
#=========================
#== L35 ==================
f1      = netCDF4.Dataset('FORTE_MHT_GMDplot.nc')
MHT     = f1.variables['MHT'][:]
MHTA    = f1.variables['MHTA'][:]
MHTIP   = f1.variables['MHTIP'][:]
f1.close()
#=========================
#== L20 ==================
f1      = netCDF4.Dataset('FORTE_MHT_L20_GMDplot.nc')
MHT20   = f1.variables['MHT'][:]
MHTA20  = f1.variables['MHTA'][:]
MHTIP20 = f1.variables['MHTIP'][:]
f1.close()

#=========================
# Figure 11
#=========================
#== L35 and L20 ==========
f1       = netCDF4.Dataset('FORTE_MLD_GMDplot.nc')
MLDL20   = f1.variables['MLDL20'][:]
MLDL35   = f1.variables['MLDL35'][:]
MLD_DEPS = f1.variables['MLD_DEPS'][:]
f1.close()
#=========================

# End load datasets



#==================
# Plot figures
#==================
# Figure 1
fig, (axarr) = plt.subplots(3, 2, figsize=(20,20),sharex=True)

axarr[0,0].plot(yr,hflux)
axarr[0,0].plot(yr20,hflux20)
axarr[0,0].plot(yr,hflux*0)
axarr[0,0].set_ylim([-1.6,1.6])
axarr[0,0].set_ylabel("W m$^{-2}$", fontsize=20)

axarr[0,1].plot(yr,wflux)
axarr[0,1].plot(yr20,wflux20)
axarr[0,1].plot(yr,wflux*0)
axarr[0,1].set_ylim([-3,3])
axarr[0,1].set_ylabel("mm yr$^{-1}$", fontsize=20)

axarr[1,0].plot(yr,sst)
axarr[1,0].plot(yr20,sst20)
axarr[1,0].set_ylim([18,19.5])
axarr[1,0].set_ylabel("$^{o}$C", fontsize=20)

axarr[1,1].plot(yr,sss)
axarr[1,1].plot(yr20,sss20)
axarr[1,1].set_ylim([34.8,35.3])
axarr[1,1].set_ylabel("psu", fontsize=20)

axarr[2,0].plot(yr,tav)
axarr[2,0].plot(yr20,tav20)
axarr[2,0].set_ylim([3.4,3.8])
axarr[2,0].set_xlabel("Time (years)", fontsize=20)
axarr[2,0].set_ylabel("$^{o}$C", fontsize=20)

axarr[2,1].plot(yr,sav)
axarr[2,1].plot(yr20,sav20)
axarr[2,1].set_ylim([34.728,34.732])
axarr[2,1].set_xlabel("Time (years)", fontsize=20)
axarr[2,1].set_ylabel("psu", fontsize=20)
plt.subplots_adjust(wspace=0.25)

# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(-0.18, 1.07, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes, 
	            size=20)
    ax.tick_params(labelsize=16)

fig.savefig('Figures/FORTE2_Figure01_R1.pdf', dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)

#Figure 2
#### Choose colormap... 
mycmap = plt.cm.get_cmap("seismic")# jet, seismic, spectral
# extract colors from the .jet map and set over/under to be same as max/min
cmaplist = [mycmap(i) for i in range(mycmap.N)]
mycmap.set_under(cmaplist[0])
mycmap.set_over(cmaplist[-1])

c = mcolors.ColorConverter().to_rgb
mybr   = make_colormap([c('blue'), c('white'), 0.45, c('white'), .55, c('white'), c('red')])
mybrD  = make_colormap([c('royalblue'), c('white'), 0.45, c('white'), .55, c('white'), c('darkorange')])
mybrS  = make_colormap([c('darkviolet'), c('white'), 0.45, c('white'), .55, c('white'), c('seagreen')])
mybrSD = make_colormap([c('darkslateblue'), c('white'), 0.45, c('white'), .55, c('white'), c('limegreen')])

fig, (axarr) = plt.subplots(4, 2, figsize=(16,16),sharex=True)

### 
ttdsga = np.transpose(ttdsg-np.tile(ttdsg[0,:],[len(yr),1]))
stdsga = np.transpose(stdsg-np.tile(stdsg[0,:],[len(yr),1]))
zssta  = np.transpose(zsst-np.tile(zsst[0,:],[len(yr),1]))
zsssa  = np.transpose(zsss-np.tile(zsss[0,:],[len(yr),1]))

### 
ttdsga20 = np.transpose(ttdsg20-np.tile(ttdsg20[0,:],[len(yr20),1]))
stdsga20 = np.transpose(stdsg20-np.tile(stdsg20[0,:],[len(yr20),1]))
zssta20  = np.transpose(zsst20-np.tile(zsst20[0,:],[len(yr20),1]))
zsssa20  = np.transpose(zsss20-np.tile(zsss20[0,:],[len(yr20),1]))

### 
ttdsgaD = ttdsga20 - ttdsga
stdsgaD = stdsga20 - stdsga
zsstaD  = zssta20 - zssta
zsssaD  = zsssa20 - zsssa

my_cs = []

levels = np.arange(-3,3.1,.2)
cs = axarr[0,0].contourf(yr,lat,zssta,levels,cmap=mybr, extend='both'); my_cs.append(cs)
axarr[0,0].set_ylabel("Latitude", fontsize=20)

levels = np.arange(-2,2.1,.2)
cs = axarr[0,1].contourf(yr,lat,zsssa,levels,cmap=mybrS, extend='both'); my_cs.append(cs)

levels = np.arange(-1,1.1,.1)
cs = axarr[1,0].contourf(yr,lat,zsstaD,levels,cmap=mybrD, extend='both'); my_cs.append(cs)
axarr[1,0].set_ylabel("Latitude", fontsize=20)

levels = np.arange(-.4,.45,.05)
cs = axarr[1,1].contourf(yr,lat,zsssaD,levels,cmap=mybrSD, extend='both'); my_cs.append(cs)
axarr[1,1].tick_params(labelsize=18)

#

levels = np.arange(-2.2,2.3,.2)
cs = axarr[2,0].contourf(yr,dp,ttdsga,levels,cmap=mybr, extend='both'); my_cs.append(cs)
axarr[2,0].set_ylabel("Depth (m)", fontsize=20)
axarr[2,0].invert_yaxis()

levels = np.arange(-.35,.38,.05)
cs = axarr[2,1].contourf(yr,dp,stdsga,levels,cmap=mybrS, extend='both'); my_cs.append(cs)
axarr[2,1].invert_yaxis()

levels = np.arange(-.5,.51,.1)
cs = axarr[3,0].contourf(yr,dp,ttdsgaD,levels,cmap=mybrD, extend='both'); my_cs.append(cs)
axarr[3,0].set_xlabel("Time (years)", fontsize=20)
axarr[3,0].set_ylabel("Depth (m)", fontsize=20)
axarr[3,0].invert_yaxis()

levels = np.arange(-.12,.13,.02)
cs = axarr[3,1].contourf(yr,dp,stdsgaD,levels,cmap=mybrSD, extend='both'); my_cs.append(cs)
axarr[3,1].set_xlabel("Time (years)", fontsize=20)
axarr[3,1].invert_yaxis()


# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(-0.1, 1.05, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes, 
            size=20)
    ax.tick_params(labelsize=16)
					    
    cbar = fig.colorbar(mappable=my_cs[n],orientation='vertical',ax=ax,shrink=.98,pad=0.02)
    cbar.ax.tick_params(labelsize=16) 

fig.savefig('Figures/FORTE2_Figure02_R1.pdf', dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)


#Figure 3
fig, (axarr) = plt.subplots(1, 2, figsize=(20,7))

axarr[0].plot(yr,omax30)
axarr[0].plot(yr20,omax3020)
axarr[0].set_ylim([6,20])
axarr[0].tick_params(labelsize=18)
axarr[0].set_xlabel("Time (years)", fontsize=20)
axarr[0].set_ylabel("Sv", fontsize=20)

axarr[1].plot(yr,dptrans)
axarr[1].plot(yr20,dptrans20)
axarr[1].set_ylim([70,160])
axarr[1].tick_params(labelsize=18)
axarr[1].set_xlabel("Time (years)", fontsize=20)
axarr[1].set_ylabel("Sv", fontsize=20)

# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(-0.1, 1.05, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes,
	            size=20)

fig.savefig('Figures/FORTE2_Figure03_R1.pdf', dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)


# Figure 4
#==================
def Fglobarra( ax, inarray, cmin=None, cmax=None, pal='seismic', clev=6 , clab=6):

    ax.set_global()
    ax.coastlines()
    ax.gridlines()
    ax.add_feature(cartopy.feature.LAND, zorder=0)
    
    cs = ax.pcolormesh(lona ,lata[1:], inarray[1:,:], cmap=pal, vmin=cmin, vmax=cmax, transform=ccrs.PlateCarree())
    cbar = plt.colorbar(mappable=cs,orientation='horizontal',ax=ax,shrink=.6,pad=0.01, extend='both')
    cbar.ax.tick_params(labelsize=20) 

    inarray2 = np.insert(inarray,0,inarray[:,-1],axis=1) # Duplicate last column of array at start to 'join' contours at prime meridian
    cs2 = ax.contour(lona[:], lata[1:-1], inarray2[1:,:], clev, colors='k', transform=ccrs.PlateCarree())
    plt.clabel(cs2, clab,  inline=True, fontsize=16, fmt = '%1.0f')
#==================

c = mcolors.ColorConverter().to_rgb
mybr = make_colormap(
    [c('blue'), c('white'), 0.45, c('white'), .55, c('white'), c('red')])

fig, (axarr) = plt.subplots(2, 2, figsize=(19,12))
fig.tight_layout()

clev = [-15,-10,-5,0,5,10,15,20,25,30]
clab = [-15,-10,-5,0,5,10,15,20,25,30]
axarr[0,0] = plt.subplot(2, 2, 1, projection=ccrs.Robinson())
Fglobarra( axarr[0,0], STanmn, cmin=-40, cmax=40, pal="RdBu_r", clev=clev, clab=clab)

axarr[0,1] = plt.subplot(2, 2, 2, projection=ccrs.Robinson())
Fglobarra( axarr[0,1], dSAT35, cmin=-15, cmax=15, pal=mybr, clev=clev, clab=clab)

clev = [5,10,20,30,40]
clab = [5,10,20,30,40]
axarr[1,0] = plt.subplot(2, 2, 3, projection=ccrs.Robinson())
Fglobarra( axarr[1,0], np.abs(np.max(STmonclim,axis=0)-np.min(STmonclim,axis=0)), cmin=0, cmax=45, pal=mybr, clev=clev, clab=clab)

axarr[1,1] = plt.subplot(2, 2, 4, projection=ccrs.Robinson())
Fglobarra( axarr[1,1], np.abs(np.max(C20_SAT_monclim,axis=0)-np.min(C20_SAT_monclim,axis=0)), cmin=0, cmax=45, pal=mybr, clev=clev, clab=clab)

# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(-0.01, 1.01, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes,
	        size=20)

plt.savefig("Figures/FORTE2_Figure04_R1.pdf", dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)


# Figure 5
#==================
def Fglobarra( ax, inarray, cmin=None, cmax=None, pal='seismic', clev=6 , clab=6):

    ax.set_global()
    ax.coastlines()
    ax.gridlines()
    ax.add_feature(cartopy.feature.LAND, zorder=0)
    
    cs = ax.pcolormesh(lona ,lata, inarray, cmap=pal, vmin=cmin, vmax=cmax, transform=ccrs.PlateCarree())
    cbar = plt.colorbar(mappable=cs,orientation='horizontal',ax=ax,shrink=.6,pad=0.01, extend='both')
    cbar.ax.tick_params(labelsize=20) 

    inarray2 = np.insert(inarray,0,inarray[:,-1],axis=1) # Duplicate last column of array at start to 'join' contours at prime meridian
    cs2 = ax.contour(lona[:], lata[1:-1], inarray2[1:,:], clev, colors='k', transform=ccrs.PlateCarree())
    plt.clabel(cs2, clab,  inline=True, fontsize=16, fmt = '%1.0f')
#==================

c = mcolors.ColorConverter().to_rgb
rain = make_colormap([c('white'), c('blue'), 0.5, c('blue'), c('orange'), 0.9, c('orange'), c('red')])

c = mcolors.ColorConverter().to_rgb
myra = make_colormap([c('brown'), c('white'), 0.4, c('white'), .6, c('white'), c('blue')])

fig, (axarr) = plt.subplots(2, 2, figsize=(19,12))
fig.tight_layout()

clev = [5,10,20,30,40]
clab = [5,10,20,30,40]
axarr[0,0] = plt.subplot(2, 2, 1, projection=ccrs.Robinson())
Fglobarra( axarr[0,0], PTdjfclim, cmin=0, cmax=15, pal=rain, clev=clev, clab=clab)

axarr[1,0] = plt.subplot(2, 2, 3, projection=ccrs.Robinson())
Fglobarra( axarr[1,0], PTjjaclim, cmin=0, cmax=15, pal=rain, clev=clev, clab=clab)

clev = [-10,-5,0,5,10]
clab = [-10,-5,0,5,10]
axarr[0,1] = plt.subplot(2, 2, 2, projection=ccrs.Robinson())
Fglobarra( axarr[0,1], PTdjfclim-C20_Precip_djfclim, cmin=-10, cmax=10, pal=myra, clev=clev, clab=clab)

axarr[1,1] = plt.subplot(2, 2, 4, projection=ccrs.Robinson())
Fglobarra( axarr[1,1], PTjjaclim-C20_Precip_jjaclim, cmin=-10, cmax=10, pal=myra, clev=clev, clab=clab)

# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(-0.01, 1.01, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes,
            size=20)
						    
plt.savefig("Figures/FORTE2_Figure05_R1.pdf", dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)


# Figure 6
#==================
def Fglobarra( ax, inarray, cmin=None, cmax=None, pal='seismic', clev=6 , clab=6):

    ax.set_global()
    ax.coastlines()
    ax.gridlines()
    ax.add_feature(cartopy.feature.LAND, zorder=0)
				    
    cs = ax.pcolormesh(lona ,lata, inarray, cmap=pal, vmin=cmin, vmax=cmax, transform=ccrs.PlateCarree())
    cbar = plt.colorbar(mappable=cs,orientation='horizontal',ax=ax,shrink=.6,pad=0.01, extend='both')
    cbar.ax.tick_params(labelsize=20) 

    inarray2 = np.insert(inarray,0,inarray[:,-1],axis=1) # Duplicate last column of array at start to 'join' contours at prime meridian
    cs2 = ax.contour(lona[:], lata[1:-1], inarray2[1:,:], clev, colors='k', transform=ccrs.PlateCarree())
    plt.clabel(cs2, clab,  inline=True, fontsize=16, fmt = '%1.0f')
#==================

import cmocean

c = mcolors.ColorConverter().to_rgb
mybr = make_colormap(
    [c('blue'), c('white'), 0.45, c('white'), .55, c('white'), c('red')])
mybrD = make_colormap(
    [c('royalblue'), c('white'), 0.45, c('white'), .55, c('white'), c('darkorange')])

fig, (axarr) = plt.subplots(3, 2, figsize=(19,18))
fig.tight_layout()

clev = [ 990, 995, 1000, 1005, 1010, 1015, 1020]
clab = [ 990, 995, 1000, 1005, 1010, 1015, 1020]
axarr[0,0] = plt.subplot(3, 2, 1, projection=ccrs.Robinson())
Fglobarra( axarr[0,0], PSanmn, cmin=990, cmax=1020, pal=cmocean.cm.diff, clev=clev, clab=clab)


clev = [5,10,15,20,25,30,40]
clab = [5,10,15,20,25,30,40]
axarr[0,1] = plt.subplot(3, 2, 2, projection=ccrs.Robinson())
Fglobarra(axarr[0,1], np.abs(np.max(PSmonclim,axis=0)-np.min(PSmonclim,axis=0)), cmin=0, cmax=30, pal="RdBu_r", clev=clev, clab=clab)

clev = [-3,-2,-1,0,1,2,3]
clab = [-3,-2,-1,0,1,2,3]
axarr[1,0] = plt.subplot(3, 2, 3, projection=ccrs.Robinson())
Fglobarra( axarr[1,0], PSanmn20-PSanmn, cmin=-2, cmax=2, pal=mybrD, clev=clev, clab=clab)


clev = [-8,-5,-2,0,2,5,8]
clab = [-8,-5,-2,0,2,5,8]
axarr[1,1] = plt.subplot(3, 2, 4, projection=ccrs.Robinson())
Fglobarra(axarr[1,1], np.abs(np.max(PSmonclim20,axis=0)-np.min(PSmonclim20,axis=0))
                     -np.abs(np.max(PSmonclim,axis=0)-np.min(PSmonclim,axis=0))
                     , cmin=-5, cmax=5, pal=mybrD, clev=clev, clab=clab)


clev = [ 990, 995, 1000, 1005, 1010, 1015, 1020]
clab = [ 990, 995, 1000, 1005, 1010, 1015, 1020]
axarr[2,0] = plt.subplot(3, 2, 5, projection=ccrs.Robinson())
Fglobarra( axarr[2,0], PSmonclim[0,:,:], cmin=990, cmax=1020, pal=cmocean.cm.diff, clev=clev, clab=clab)

axarr[2,1] = plt.subplot(3, 2, 6, projection=ccrs.Robinson())
Fglobarra( axarr[2,1], PSmonclim[6,:,:], cmin=990, cmax=1020, pal=cmocean.cm.diff, clev=clev, clab=clab)


# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(-0.01, 1.01, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes, 
            size=20)
				    
plt.savefig("Figures/FORTE2_Figure06_R1.pdf", dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)




# Figure 7
myuv = make_colormap(
    [c('blue'), c('white'), 0.25, c('white'), c('orange'), 0.7, c('orange'), c('red')])

fig, (axarr) = plt.subplots(2, 1, figsize=(19,14))

levels = np.arange(-10,45,5)
clev = np.arange(-10,45,5)
cs = axarr[0].contourf(lata2,pres,UDJF,levels,cmap=myuv, extend='both')
cs2 = axarr[0].contour(lata2,pres,UDJF, levels, colors='k')
plt.clabel(cs2, clev.astype(int),  inline=True, fontsize=14, fmt = '%1.0f')
axarr[0].tick_params(labelsize=18)
axarr[0].set_xlabel("Latitude", fontsize=20)
axarr[0].set_ylabel("Pressure (hPa)", fontsize=20)
axarr[0].invert_yaxis()
axarr[0].set_xlim(-90,90)
cbar = fig.colorbar(mappable=cs,orientation='vertical',ax=axarr[0],shrink=.8,pad=0.02)
cbar.ax.tick_params(labelsize=20) 

cs = axarr[1].contourf(lata2,pres,UJJA,levels,cmap=myuv, extend='both')
cs2 = axarr[1].contour(lata2,pres,UJJA, levels, colors='k')
plt.clabel(cs2, clev.astype(int),  inline=True, fontsize=14, fmt = '%1.0f')
axarr[1].tick_params(labelsize=18)
axarr[1].set_xlabel("Latitude", fontsize=20)
axarr[1].set_ylabel("Pressure (hPa)", fontsize=20)
axarr[1].invert_yaxis()
axarr[1].set_xlim(-90,90)
cbar = fig.colorbar(mappable=cs,orientation='vertical',ax=axarr[1],shrink=.8,pad=0.02)
cbar.ax.tick_params(labelsize=20) 

cs2 = axarr[0].contour(lata2,press20,UDJF20,levels, colors='dimgray')

cs2 = axarr[1].contour(lata2,press20,UJJA20,levels, colors='dimgray')


# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(-0.1, 1.05, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes,
	            size=20)

fig.savefig('Figures/FORTE2_Figure07_R1.pdf', dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)


# Figure 8
#==================
def Fglobarr( ax, inarray, cmin=None, cmax=None, pal='seismic', clev=6 , clab=6):

    from cartopy.util import add_cyclic_point
    ax.set_global()
    ax.coastlines()
    ax.gridlines()
    ax.add_feature(cartopy.feature.LAND, zorder=0)
    inarrayc = add_cyclic_point(inarray)
    lonsc = np.append(lon,360.99)
		
    cs = ax.pcolormesh(lonu, latu, inarray, cmap=pal, vmin=cmin, vmax=cmax, transform=ccrs.PlateCarree())
    cbar = plt.colorbar(mappable=cs,orientation='horizontal',ax=ax,shrink=.6,pad=0.01, extend='both')
    cbar.ax.tick_params(labelsize=20) 

    cs2 = ax.contour(lonsc ,lat, inarrayc, clev, colors='k', transform=ccrs.PlateCarree())
    plt.clabel(cs2, clab,  inline=True, fontsize=16, fmt = '%1.0f')
#==================

c = mcolors.ColorConverter().to_rgb
mybr = make_colormap(
    [c('blue'), c('white'), 0.45, c('white'), .55, c('white'), c('red')])
mybrD = make_colormap(
    [c('royalblue'), c('white'), 0.45, c('white'), .55, c('white'), c('darkorange')])
mybrS = make_colormap(
    [c('darkviolet'), c('white'), 0.45, c('white'), .55, c('white'), c('seagreen')])
mybrSD = make_colormap(
    [c('darkslateblue'), c('white'), 0.45, c('white'), .55, c('white'), c('limegreen')])


fig, (axarr) = plt.subplots(4, 2, figsize=(19,24))
fig.tight_layout()

clev = [-2,2,6,10,14,18,22,26]
clab = [2,10,18,26]
axarr[0,0] = plt.subplot(4, 2, 1, projection=ccrs.Robinson())
Fglobarr( axarr[0,0], sst2d, cmin=-2, cmax=28, pal="RdBu_r", clev=clev, clab=clab)

axarr[1,0] = plt.subplot(4, 2, 3, projection=ccrs.Robinson())
Fglobarr( axarr[1,0], esst2d, cmin=-2, cmax=28, pal="RdBu_r", clev=clev, clab=clab)

clev = [30,32,33,34,35,36]
clab = [30,32,33,34,35,36]
axarr[0,1] = plt.subplot(4, 2, 2, projection=ccrs.Robinson())
Fglobarr( axarr[0,1], sss2d, cmin=30, cmax=37, pal="BrBG_r", clev=clev, clab=clab)

axarr[1,1] = plt.subplot(4, 2, 4, projection=ccrs.Robinson())
Fglobarr( axarr[1,1], esss2d, cmin=30, cmax=37, pal="BrBG_r", clev=clev, clab=clab)

clev = list(np.arange(-7,7.1,2))
clab = list(np.arange(-5,5.1,2))
axarr[2,0] = plt.subplot(4, 2, 5, projection=ccrs.Robinson())
Fglobarr( axarr[2,0], sst2d-esst2d, cmin=-7, cmax=7, pal=mybrD, clev=clev, clab=clab)

clev = list(np.arange(-3.5,3.6,1))
clab = list(np.arange(-2.5,2.6,1))
axarr[2,1] = plt.subplot(4, 2, 6, projection=ccrs.Robinson())
Fglobarr( axarr[2,1], sss2d-esss2d, cmin=-3, cmax=3, pal=mybrSD, clev=clev, clab=clab)

clev = list(np.arange(2,2,1))
clab = list(np.arange(2,2,1))
axarr[3,0] = plt.subplot(4, 2, 7, projection=ccrs.Robinson())
Fglobarr( axarr[3,0], sst2d20-sst2d, cmin=-3, cmax=3, pal=mybrD, clev=clev, clab=clab)

clev = list(np.arange(1,1,1))
clab = list(np.arange(1,1,1))
axarr[3,1] = plt.subplot(4, 2, 8, projection=ccrs.Robinson())
Fglobarr( axarr[3,1], sss2d20-sss2d, cmin=-1, cmax=1, pal=mybrSD, clev=clev, clab=clab)

# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(-0.01, 1.01, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes,
	            size=20)

plt.savefig("Figures/FORTE2_Figure08_R1.pdf", dpi=200, transparent=True, bbox_inches='tight', pad_inches=0.1)


# Figure 9
#==================
def Fglobarr( ax, inarray, cmin=None, cmax=None, pal='seismic', clev=6 , clab=6):

    from cartopy.util import add_cyclic_point
    ax.set_global()
    ax.coastlines()
    ax.gridlines()
    ax.add_feature(cartopy.feature.LAND, zorder=0)
    inarrayc = add_cyclic_point(inarray)
    lonsc = np.append(lon,360.99)
		
    cs = ax.pcolormesh(lonu ,latu, inarray, cmap=pal, vmin=cmin, vmax=cmax, transform=ccrs.PlateCarree())
    cbar = plt.colorbar(mappable=cs,orientation='horizontal',ax=ax,shrink=.6,pad=0.01, extend='both')
    cbar.ax.tick_params(labelsize=20) 

    cs2 = ax.contour(lonsc ,lat, inarrayc, clev, colors='k', transform=ccrs.PlateCarree())
    plt.clabel(cs2, clab,  inline=True, fontsize=16, fmt = '%1.1f')
#==================

c = mcolors.ColorConverter().to_rgb
mybr = make_colormap(
    [c('blue'), c('white'), 0.64, c('white'), .65, c('white'), c('red')])

fig, (axarr) = plt.subplots(2, 2, figsize=(19,12))
fig.tight_layout()

clev = np.arange(-1.4,1.2,.2)
clab = np.arange(-1.4,1.2,.2)
axarr[0,0] = plt.subplot(2, 2, 1, projection=ccrs.Robinson())
Fglobarr( axarr[0,0], ssh, cmin=-1.8, cmax=1, pal=mybr, clev=clev, clab=clab)
axarr[0,1] = plt.subplot(2, 2, 2, projection=ccrs.Robinson())
Fglobarr( axarr[0,1], occassh, cmin=-1.8, cmax=1, pal=mybr, clev=clev, clab=clab)

axarr[1,0] = plt.subplot(2, 2, 3, projection=ccrs.Robinson())
Fglobarr( axarr[1,0], ssh20-ssh, cmin=-.2, cmax=.2, pal=mybrD, clev=clev, clab=clab)
axarr[1,1] = plt.subplot(2, 2, 4, projection=ccrs.Robinson())
Fglobarr( axarr[1,1], ssh-occassh, cmin=-1, cmax=1, pal=mybrD, clev=clev, clab=clab)

# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(-0.01, 1.01, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes,
	            size=20)

plt.savefig("Figures/FORTE2_Figure09_R1.pdf", dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)


# Figure 10
#### Choose colormap... 
mycmap = plt.cm.get_cmap("viridis")# jet, seismic, spectral
# extract colors from the .jet map and set over/under to be same as max/min
cmaplist = [mycmap(i) for i in range(mycmap.N)]
mycmap.set_under(cmaplist[0])
mycmap.set_over(cmaplist[-1])

amocm  = np.mean(amoc[380:,:,:],axis=0)
amocm20  = np.mean(amoc20[380:,:,:],axis=0)

fig, (axarr) = plt.subplots(2, 1, figsize=(19,16),sharex=True)


levels = np.arange(-4,19,1)
cs = axarr[0].contourf(lat[28:],dp,amocm,levels,cmap=mycmap, extend='both')
axarr[0].tick_params(labelsize=18)
axarr[0].set_ylabel("Depth (m)", fontsize=20)
axarr[0].invert_yaxis()
axarr[0].set_xlim(-30,70)
cbar = fig.colorbar(mappable=cs,orientation='vertical',ax=axarr[0],shrink=.98,pad=0.02)
#plt.subplots_adjust(wspace=0.1, hspace=0.1)
cbar.ax.tick_params(labelsize=20) 

levels = np.arange(-4,19,1)
cs = axarr[1].contourf(lat[28:],dp,amocm20,levels,cmap=mycmap, extend='both')
axarr[1].tick_params(labelsize=18)
axarr[1].set_xlabel("Latitude", fontsize=20)
axarr[1].set_ylabel("Depth (m)", fontsize=20)
axarr[1].invert_yaxis()
axarr[1].set_xlim(-30,70)
cbar = fig.colorbar(mappable=cs,orientation='vertical',ax=axarr[1],shrink=.98,pad=0.02)
#plt.subplots_adjust(wspace=0.1, hspace=0.1)
cbar.ax.tick_params(labelsize=20) 

# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(-0.05, 1.05, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes, 
	        size=20)

fig.savefig('Figures/FORTE2_Figure10_R1.pdf', dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)


#Figure 11
#==================
def Fglobarr( ax, inarray, cmin=None, cmax=None, pal='seismic', clev=6 , clab=6):

    from cartopy.util import add_cyclic_point

    ax.set_global()
    ax.coastlines()
    ax.gridlines()
    ax.add_feature(cartopy.feature.LAND, zorder=0)
    inarrayc = add_cyclic_point(inarray)
    lonsc = np.append(lon,360.99)
							    
    cs = ax.pcolormesh(lonu, latu, inarray, cmap=pal, vmin=cmin, vmax=cmax, transform=ccrs.PlateCarree())

    import matplotlib as mpl

    mycmap = plt.cm.get_cmap(pal)
    cmaplist = [mycmap(i) for i in range(mycmap.N)]
    # create the new map
    mycmap = mpl.colors.LinearSegmentedColormap.from_list('Custom cmap', cmaplist, mycmap.N)
		    
    vlmin = 0
    vlmax = 9
    ccmax = 9
    # define the bins and normalize
    bounds = np.linspace(vlmin, vlmax, ccmax+1)
    norm = mpl.colors.BoundaryNorm(bounds, mycmap.N)

    fs = 16

    axc1 = fig.add_axes([.8, 0.125, 0.016, 0.752])
    norm = mpl.colors.Normalize(vmin=cmin,vmax=cmax)
    cb1  = mpl.colorbar.ColorbarBase(axc1, cmap=mycmap, norm=norm, 
    orientation='vertical',extend='max',ticks=bounds, boundaries=bounds)
    cb1.ax.set_yticklabels(MLD_DEPS)
    cb1.ax.tick_params(labelsize=fs) 
    #==================

c = mcolors.ColorConverter().to_rgb
mybr = make_colormap(
    [c('blue'), c('white'), 0.45, c('white'), .55, c('white'), c('red')])

fig, (axarr) = plt.subplots(2, 1, figsize=(16,12))

clev = [-2,2,6,10,14,18,22,26]
clab = [2,10,18,26]
axarr[0] = plt.subplot(2, 1, 1, projection=ccrs.Robinson())
Fglobarr( axarr[0], MLDL35, cmin=0, cmax=8, pal="cividis", clev=clev, clab=clab)

axarr[1] = plt.subplot(2, 1, 2, projection=ccrs.Robinson())
Fglobarr( axarr[1], MLDL20, cmin=0, cmax=8, pal="cividis", clev=clev, clab=clab)

# Label subfigures
axs = axarr.flat
for n, ax in enumerate(axs):
    ax.text(0.02, 1.05, "("+string.ascii_lowercase[n]+")", transform=ax.transAxes, 
            size=20)
				    
fig.savefig('Figures/FORTE2_Figure11_R1.pdf', dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)



#Figure 12

def smooth(y, box_pts):
    box = np.ones(box_pts)/box_pts
    y_smooth = np.convolve(y, box, mode='same')
    return y_smooth

fig,ax = plt.subplots(1, 1, figsize=(10,5))

plt.plot(latu[1:], smooth(MHTIP20,5)/1e15,color="orange",ls="--")
plt.plot(latu[1:], smooth(MHTIP,5)/1e15,label="Indo-Pacific",lw=2,color="orange")

plt.plot(latu[1:], smooth(MHT20,5)/1e15,color="green",ls="--")
plt.plot(latu[1:], smooth(MHT,5)/1e15,label="Total",lw=2,color="green")

plt.plot(latu[1:], smooth(MHTA20,5)/1e15,color="blue",ls="--")
plt.plot(latu[1:], smooth(MHTA,5)/1e15,label="Atlantic",lw=2,color="blue")

plt.plot(latu[1:], MHT*0,color='k')
plt.xlabel("Latitude")
plt.ylabel("Heat transport (PW)")
plt.xlim([-80,80])
plt.legend()

fig.savefig('Figures/FORTE2_Figure12_R1.pdf', dpi=300, transparent=True, bbox_inches='tight', pad_inches=0.1)
